*****************************************************************************
* Valid.PRG  Version 1.0 Beta                                                               *
*   By Craig M. Walsh, 8/9/1989.                                            *
*   Includes functions for nested reads/enhanced validation control.        *
*   Note: EXACT must be SET OFF; may cause serious problems!                *
*   Escape only supports one.                                               *
*****************************************************************************


*********************FULLSCREEN VALIDATION FUNCTIONS********************

************
function vbr      &&Validation branching function.
                  &&  This function is called by the valid statement, and
                  &&  branches control to the proper valid function.
                  &&  This function maintains GET positioning for proper
                  &&  nested read functions as well as insuring proper
                  &&  validation consistency.
                  &&    Requires:
                  &&      1.  valgets - An array that includes names of all
                  &&                    GETs in the pending read area.  This
                  &&                    should be declared private.
                  &&      2.  valholds- A private array that holds the values of each
                  &&                    of the various GETs prior to editing.
                  &&      3.  valretn - An array that holds return and control
                  &&                    values.
                  &&          valretn[1] = looping control
                  &&          valretn[2] = getname to find
                  &&          valretn[3] = return status, A(bort), U(n-) or M(odified)
                  &&    Parameters:
                  &&      1.  v_udf   - Validation udf and its parameters
                  &&                    to branch to upon validation.
                  &&

  parameters v_udf
  private v_valid, v_get, v_key, v_retn

  *Check for GET-termination keys.  (See vloop for read-termination keys.)
  v_key = lastkey()
  do case

    case v_key = 5   &&<Up> Move up without validation.
      return (.t.)   &&  NOTE: READEXIT() should be set to .f.

    case v_key = 24  &&<Dn> Move down with validation.

    case v_key = 3   &&<PgDn> Terminate READ if all pending
                     &&       GETs have been validated.
    **  valretn[1] = 'T'   &&NOT YET OPERATIONAL!!!
      return (.f.)

    case v_key = 18  &&<PgUp> Move to top of READ list without validating current GET.
      valretn[1] = 'B'  &&Beginning of READ.
      return (.t.)

  endcase

  v_get = readvar()

  *Check for READ termination.
  if valretn[1] = 'T'  &&Terminate read.  Validate from active GET -> down.
    v_retn = vchk(v_get, v_udf)
    if v_retn
      valretn[1] = 'N'
      keyboard chr(13)  &&If active GET is valid, move to next GET's valid.
      return (.f.)      &&Return .f. so that <PgDn> doesn't terminate READ.
    else
      valretn[1] = 'E'
      return (.f.)
    endif
  endif

  *Check for next GET (part of READ termination.)
  if valretn[1] = 'N'
    valretn[1] = 'T'
    keyboard chr(13)
    return (.t.)
  endif

  *Check for READ jump.
  if valretn[1] = 'J'
    *Jump to specified GET.     NOTE: To stop the cursor at the current GET after
    *                           a READ loop (i.e. after nesting), will only
    *                           do so if the clause is not valid, or the
    *                           validation status has been set to invalidated.
    *
    if valretn[2] = v_get                 &&Is this the proper GET?

      *GET has been found.
      valretn[1] = ''
      valretn[2] = ''
      *Continue past this field to next one if current field has been checked.
      if vgcheck(vpos(v_get))
        keyboard chr(13)
      endif
      return (.f.)

    else

      keyboard chr(13)                    &&Goto next GET.
      return (.t.)

    endif

  else
    *Validate current field.
    valretn[1] = 'E'
    v_retn = vchk(v_get, v_udf)
    return ( v_retn )
  endif




**************
function vloop    &&Control branch point after READ.  This controls
                  &&  the do-while holding the GET statements to allow
                  &&  exit or looping (and re-reading).
  do case

    case lastkey() = 27      &&Abort editing.
      inkey(.01)             &&Get rid of chr(27) in key buffer so that
                             &&  ESC is not global.
      valretn[3] = 'A'
      return (.f.)

    case valretn[1] = 'A'    &&Abort this read.
      valretn[3] = 'A'
      return (.f.)

    case valretn[1] = 'J'    &&Jump back up!
      return (.t.)

    case valretn[1] = 'B'    &&Move to top.
      valretn[1] = 'E'
      return (.t.)

    otherwise
      return (.f.)           &&Exit loop!

  endcase


***************
function vset   &&Sets up the validation status array for each GET of the
                  &&  defined read area.
                  &&
                  &&  Requires:
                  &&    1. The array valgets must be declared (private) prior
                  &&       to invocation of this function.  This array must
                  &&       be declared with enough elements to hold all of the
                  &&       GET information; if not, the function will fail.
                  &&  Parameters:
                  &&    1. v_get - This is the GET name var to be included in
                  &&               the validation array.  This must be the
                  &&               exact same name as the GET name variable
                  &&               as issued during the GET for this GET.
                  &&  Note: In valgets[n]
                  &&    First 10 bytes of element are getname,
                  &&    byte 11 is the validation checked status (N or Y), and
                  &&    byte 12 is the GET modification status (N or Y).
                  &&  Note: valholds[n] is loaded with initial GET value.

  parameters v_get
  private v_inc, v_valid, v_mod

  v_valid = 'N'
  v_mod = 'N'

  *Initialize arrays if first call.
  if type('valgets[1]') = 'U'
    afill(valgets, '?')
    afill(valholds, '')
    valretn[1] = 'E'
    valretn[2] = ''
    valretn[3] = 'U'
  endif

  *Find next blank element.
  v_inc = vpos('?')
  if v_inc = 0
    @ 0,0 say 'Valid init failed during positioning of pointer, line 209.'
    inkey(0)
    return (.f.)   &&Init failed because too many GETs for this array.
  endif

  *Add to array elements.
  valgets[v_inc] = vpad(upper(v_get), 10) + v_valid + v_mod
  valholds[v_inc] = &v_get.

  return (.t.)




********************************INTERNAL FUNCTIONS***************************

***************
function vchk   &&Performs valid check using defined UDF.  Returns value
                  &&  of that UDF upon termination; optionally sets the
                  &&  looping structure to return to this field if there
                  &&  was a cleared GET.

  parameters v_vget, v_vudf
  private v_inc, v_vvalid, v_checked, v_modified

  *Find proper element pointer for this GET.
  v_inc = vpos(v_vget)

  *Has this GET been valid checked or modified?
  v_modified = vgmod(v_get, v_inc)
  v_checked = vgcheck(v_inc)

  *Do we need to validate this field?
  if (v_modified) .or. (.not. v_checked)

    if .not. empty(v_vudf)
      *Execute UDF or defined valid expression.
      if &v_vudf.
        v_vvalid = .t.
      else
        v_vvalid = .f.
      endif
    else
      v_vvalid = .t.
    endif

    *If valid is true, Mark that this GET has been valid checked (and possibly modified).
    if v_vvalid
      valgets[v_inc] = substr(valgets[v_inc], 1, 10) + 'Y' +;
        if(v_modified, 'Y', 'N')

      if v_modified
        valretn[3] = 'M'
      endif

      *If validation is ok, place this new GET value into save array.
      valholds[v_inc] = &v_vget.
    else
      *Make an obnoxious noise to signal that valid check has failed!
      tone(1000, .5)
      tone(100, .5)
    endif

  else

    v_vvalid = .t.

  endif

  *Return control to VBR() if this is a jump.
  if valretn[1] = 'J'
    keyboard chr(13)
  endif

  return (v_vvalid)


*************
function vpos     &&Returns the proper position in the validation arrays for specified GET.
                  &&  Parameters:
                  &&    1. v_find -  GET name to find element.
  parameters v_find
  return ( ascan(valgets, v_find) )


**************
function vgmod    &&Checks for GET modification.
  parameters v_getname, v_element
  return ( if(&v_getname. # valholds[v_element], .t., .f.) )


****************
function vgcheck  &&Has the GET been valid checked yet?
  parameters v_element
  return ( if(substr(valgets[v_element], 11, 1) = 'Y', .t., .f.) )


***************
function vpad     &&Pad a string with specified number of trailing blanks.
                  &&  Parameters:
                  &&    1. v_str  -  Input string.
                  &&    2. v_len  -  Length of padded return string.
  parameters v_str, v_len
  return ( substr(v_str, 1) + space(if(len(trim(v_str)) > 0, v_len - len(trim(v_str)), 0)) )



*******************UTILITY FUNCTIONS FOR NESTED READS, ETC.******************

******************
function cleargets &&Clear the pending GET screen for a nested screen.
                   &&Sets up for a return to the GET where the nesting occured.
  private f_get
  f_get = readvar()
  valretn[1] = 'J'
  valretn[2] = f_get
  clear gets
  return (.t.)


****************
function voldget   &&Returns the value of the GET prior to editing and validation.
                   &&  Parameters:
                   &&    1. v_get -  GETname to use in lookup.
  parameters v_get
  return ( valholds[vpos(v_get)] )


**************
function vquit     &&Terminate pending GETs and return control
                   && to commands below calling read loop.
  clear gets
  valretn[1] = 'A'
  valretn[3] = 'A'
  return (.t.)



****************
function vjump    &&Sets the validation routine to jump to the defined GETname
                  &&upon return of control to VBR.  Note that the active GET must
                  &&be validated (true) upon return.
                  &&  Parameters:
                  &&    1. v_get -  GET name to jump to.
                  &&
                  &&  Usage:
                  &&    This function is called during a GET validation UDF
                  &&    to allow placement at another GET upon return of
                  &&    control.  Thus, it allows non-sequential control
                  &&    of GET order.
  parameters v_get
  valretn[1] = 'J'
  valretn[2] = upper(alltrim(v_get))
  clear gets
  return (.t.)


***************
function vabort   &&Abort status of last read.
  if valretn[3] = 'A'
    return (.t.)
  else
    return (.f.)
  endif


*****************
function vupdated &&Returns modification status for a specified GET. (Like the update() fxn).
                  &&  Use this function after read termination to determine
                  &&  if a particular GET has been edited.  If you wish
                  &&  to determine if the screen has been modified, just
                  &&  check the return value in valretn.  This function is
                  &&  just for checking the status for a particular GET.
                  &&    Parameters:
                  &&      1. v_get -  GET to check for modification.
                  &&    Returns:
                  &&      1. .t.    -  Modified.
                  &&      2. .f.    -  Unmodified.
  parameters v_get
  return ( if(substr(valget[vpos(v_get)], 12, 1)='M', .t., .f.) )
