****************************************************************************
**ValiDemo.prg                                                              *
*   Program to demonstrate the use of a validation branching UDF to        *
*     control nested reads and validation consistency.                     *
* AUTHOR:  Craig M. Walsh                                                  *
* CREATED: 08/09/1989                                                      *
*                                                                          *
****************************************************************************
set procedure to valid
clear screen
private mNAME, mACCOUNT, mADDRESS, mADDRESS2, mCITY, mSTATE, mZIP, mPHONE, scrn1_stat

set escape on  &&Enables escape from screens.

*Main program loop.
do while .t.
  editinit()
  scrn1_stat = screen1()
  do case

    case scrn1_stat = 'A'
      @ 23, 0 say 'Account screen edits aborted!'
      inkey(0)
      exit

    case scrn1_stat = 'M'
      @ 23, 0 say 'Account screen modified!'
      inkey(0)
      loop

    case scrn1_stat = 'U'
      @ 23, 0 say 'Account screen has not been modified!'
      inkey(0)
      loop

    otherwise
      @ 23, 0 say 'Return value was: '+scrn1_stat
      inkey(0)
      exit

  endcase

enddo while .t.
return





*******************Main program functions*********************************
function editinit   &&Load the gets for the primary screen.
  mNAME = space(30)
  mACCOUNT = space(6)
  mADDRESS = space(30)
  mADDRESS2 = space(30)
  mCITY = space(22)
  mSTATE = space(2)
  mZIP = space(10)
  mPHONE = space(14)
  return (.t.)


function screen1  &&Primary input screen.
  private valgets[8], valholds[8], valretn[3]
  @ 1, 0 clear to 10, 45
  @ 1, 0 to 10, 45 double
  @ 1, 17 say '<CLIENT>'
  vset('mNAME')
  vset('mACCOUNT')
  vset('mADDRESS')
  vset('mADDRESS2')
  vset('mCITY')
  vset('mSTATE')
  vset('mZIP')
  vset('mPHONE')
  do while .t.
    @  2,  2 say 'NAME:     ' get mNAME valid vbr('v_name(mNAME)')
    @  3,  2 say 'ACCOUNT:  ' get mACCOUNT valid vbr('v_account(@mACCOUNT)')  &&Note use of '@' which
                                                                              && allows return of value to calling GET.
    @  4,  2 say 'ADDRESS:  ' get mADDRESS valid vbr('.not. empty(mADDRESS)')
    @  5,  2 say 'ADDRESS2: ' get mADDRESS2 valid vbr('')
    @  6,  2 say 'CITY:     ' get mCITY valid vbr('v_city(@mCITY)')
    @  7,  2 say 'STATE:    ' get mSTATE picture '!!' valid vbr('')
    @  8,  2 say 'ZIP:      ' get mZIP valid vbr('.not. empty(mZIP)')
    @  9,  2 say 'PHONE:    ' get mPHONE valid vbr('v_phone(@mPHONE)')
    read
    if .not. vloop()
      exit
    endif
  enddo
  return ( valretn[3] )

***************************************************************************
*                  FIELD VALID FUNCTIONS                                  *
***************************************************************************

*NOTE:  Since these functions are called by macro substitution, there
*       is no checking for them during compile or link.  Thus, it is
*       imperative that you ensure that all included UDFs for a screen
*       are included somewhere in your included objects.  The UDFs for a screen
*       can be included in the same file, or alternately, in a file called
*       VALID.PRG which is linked to the rest of the executable during
*       linkage.  Make sure to test all valid expressions and UDFs
*       extensively during development to ensure that they function as they
*       should.  VBR() does not perform any error checking, nor does VALSET().

function v_name  &&Validate the name field of screen1.
  parameters f_name
  if empty(f_name)
    @23, 0 say 'Name field cannot be blank!'
    return (.f.)
  else
    @23, 0 clear to 23, 79
    return (.t.)
  endif



function v_account   &&Validate the account number for screen1.
                     &&Example of a nested read.
  parameters f_account
  if .not. empty(f_account)
    return (.t.)
  endif

  *Set up screen2 for getting new account info.

  cleargets()     &&Clear the current screen1 GETs to invoke screen2 read.
                  &&Note that this is a special UDF in VALID.PRG!!!

  private mCARRIER, mTYPE, mYEAR
  mCARRIER = space(3)
  mTYPE = space(1)
  mYEAR = space(2)

  @ 1, 48 clear to 10, 79
  @ 1, 48 to 10, 79 double
  @ 1, 60 say '<ACCOUNT>'

  private valgets[3], valholds[3], valretn[3]
  vset('mCARRIER')
  vset('mTYPE')
  vset('mYEAR')

  @ 23, 0 say 'Enter account information...'
  do while .t.
    @ 2, 50 say 'CARRIER: ' get mCARRIER picture '!!!' valid vbr('v_carrier(@mCARRIER)')
    @ 3, 50 say 'TYPE:    ' get mTYPE    picture '!'   valid vbr('.not. empty(mTYPE)')
    @ 4, 50 say 'YEAR:    ' get mYEAR    picture '99'  valid vbr('.not. empty(mYEAR)')
    read
    if .not. vloop()
      exit
    endif
  enddo
  @ 1, 48 clear to 10, 79
  @ 23, 0 clear to 23, 79
  if vabort()
    return (.f.)
  else
    f_account = mCARRIER + mYEAR + mTYPE  &&Return new account to screen1.
    return (.t.)
  endif


function v_city  &&Validate CITY fields.
  parameters f_city
  if empty(f_city)
    return (.f.)
  endif
  do case

    case upper(trim(f_city)) == 'NY'
      f_city = 'New York'
      mSTATE = 'NY'
      vjump('mZIP')

    case upper(trim(f_city)) == 'LA'
      f_city = 'Los Angeles'
      mSTATE = 'CA'
      vjump('mZIP')

  endcase
  return (.t.)



function v_carrier  &&Validate the carrier field of screen2.  This is a
                    &&  screen nested to the third level.

  parameters v_code
  if .not. empty(v_code)
    return (.t.)
  endif

  *Set up screen3.
  @ 12, 0 clear to 15, 45
  @ 12, 0 to 15, 45 double
  @ 12, 20 say '<CARRIER>'

  cleargets()    &&Note that this must come before the PRIVATE command!

  private valgets[2], valholds[2], valretn[3]
  private  mCOMPANY, mBRANCH
  mCOMPANY = space(10)
  mBRANCH = space(10)
  vset('mCOMPANY')
  vset('mBRANCH')
  do while .t.
    @ 13, 2 say 'Company:  ' get mCOMPANY valid vbr('.not. empty(mCOMPANY)')
    @ 14, 2 say 'Branch:   ' get mBRANCH  valid vbr('.not. empty(mBRANCH)')
    read
    if .not. vloop()
      exit
    endif
  enddo
  @ 12,0 clear to 15, 45
  if vabort()
    return (.f.)
  else
    v_code = upper(substr(mCOMPANY, 1, 2)) + upper(substr(mBRANCH, 1, 1))
    return (.t.)
  endif


function v_phone   &&Validate screen1 phone field.
  parameters phone_num
  private f_alt
  f_alt = phoneval(phone_num)
  if f_alt # phone_num
    vjump('mPHONE')            &&Jump only if number format is altered
    phone_num = f_alt          &&  for redisplay purpose.
  endif
  return (.t.)


function phoneval  &&Generic phone number validation.
                   &&Returns PH in correct format.
                   &&If # = '8265057' or '826-5057'
                   &&   returns '(213) 826-5057'.
                   &&If # = '2138265057',
                   &&   returns '(213) 826-5057'.
                   &&If # = '213-826-5057'
                   &&   returns '(213) 826-5057'.
                   &&
                   && Parameters:
                   &&  1. f_phone - Phone number string.
  parameters f_phone
  private f_alt
  f_alt = alltrim(f_phone)

  *Remove all '()-' from number.
  f_alt = strtran(f_alt, '(','')
  f_alt = strtran(f_alt, ')','')
  f_alt = strtran(f_alt, '-','')
  f_alt = strtran(f_alt, ' ','')

  *Return phone in proper format depending on length of modified string.
  do case

    case len(f_alt) = 7
      f_phone = '(213) '+ substr(f_alt, 1, 3) + ;
        '-' + substr(f_alt, 4, 4)

    case len(f_alt) = 10
      f_phone = '(' + substr(f_alt, 1, 3) + ') '+ ;
        substr(f_alt, 4, 3) + '-' + substr(f_alt, 7, 4)

  endcase
  return (f_phone)
